'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.toDynamodbRecords = exports.update = exports.timestampCondition = exports.updateExpression = exports.ttl = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; /* eslint import/no-extraneous-dependencies: ["error", {"devDependencies": true}] */


var _awsSdk = require('aws-sdk');

var aws = _interopRequireWildcard(_awsSdk);

var _highland = require('highland');

var _highland2 = _interopRequireDefault(_highland);

var _merge = require('lodash/merge');

var _merge2 = _interopRequireDefault(_merge);

var _dynamodb = require('../connectors/dynamodb');

var _dynamodb2 = _interopRequireDefault(_dynamodb);

var _faults = require('./faults');

var _print = require('./print');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

const ttl = exports.ttl = (start, days) => start / 1000 + 60 * 60 * 24 * days;

const updateExpression = exports.updateExpression = Item => ({
  ExpressionAttributeNames: Object.keys(Item).map(attrName => ({ [`#${attrName}`]: attrName })).reduce(_merge2.default, {}),
  ExpressionAttributeValues: Object.keys(Item).map(attrName => ({ [`:${attrName}`]: Item[attrName] })).reduce(_merge2.default, {}),
  UpdateExpression: `SET ${Object.keys(Item).map(attrName => `#${attrName} = :${attrName}`).join(', ')}`,
  ReturnValues: 'ALL_NEW'
});

const timestampCondition = exports.timestampCondition = (fieldName = 'timestamp') => ({
  ConditionExpression: `attribute_not_exists(#${fieldName}) OR #${fieldName} < :${fieldName}`
});

const update = exports.update = ({
  debug = (0, _print.debug)('dynamodb'),
  tableName = process.env.ENTITY_TABLE_NAME,
  updateRequestField = 'updateRequest',
  parallel = Number(process.env.UPDATE_PARALLEL) || Number(process.env.PARALLEL) || 4
} = {}) => {
  const connector = new _dynamodb2.default({ debug, tableName });

  const invoke = uow => {
    const p = connector.update(uow[updateRequestField]).then(updateResponse => _extends({}, uow, { updateResponse })).catch((0, _faults.rejectWithFault)(uow));

    return (0, _highland2.default)(p); // wrap promise in a stream
  };

  return s => s.map(invoke).parallel(parallel);
};

// testing
const toDynamodbRecords = exports.toDynamodbRecords = events => ({
  Records: events.map((e, i) => ({
    eventID: `${i}`,
    eventName: !e.oldImage ? 'INSERT' : !e.newImage ? 'REMOVE' : 'MODIFY', // eslint-disable-line no-nested-ternary
    // eventVersion: '1.0',
    eventSource: 'aws:dynamodb',
    awsRegion: 'us-west-2',
    dynamodb: {
      ApproximateCreationDateTime: e.timestamp,
      Keys: e.keys ? aws.DynamoDB.Converter.marshall(e.keys) : /* istanbul ignore next */undefined,
      NewImage: e.newImage ? aws.DynamoDB.Converter.marshall(e.newImage) : undefined,
      OldImage: e.oldImage ? aws.DynamoDB.Converter.marshall(e.oldImage) : undefined,

      SequenceNumber: `${i}`,
      // SizeBytes: 59,
      StreamViewType: 'NEW_AND_OLD_IMAGES'
    }
    // eventSourceARN: 'arn:aws:dynamodb:us-west-2:123456789012:table/myservice-entities/stream/2016-11-16T20:42:48.104',
  }))
});