'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.toKinesisRecords = exports.envTags = exports.adornStandardTags = exports.toRecord = exports.publish = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _highland = require('highland');

var _highland2 = _interopRequireDefault(_highland);

var _kinesis = require('../connectors/kinesis');

var _kinesis2 = _interopRequireDefault(_kinesis);

var _filters = require('../filters');

var _batch = require('./batch');

var _faults = require('./faults');

var _print = require('./print');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const publish = exports.publish = ({
  debug = (0, _print.debug)('kinesis'),
  streamName = process.env.STREAM_NAME,
  eventField = 'event',
  batchSize = Number(process.env.PUBLISH_BATCH_SIZE) || Number(process.env.BATCH_SIZE) || 25,
  parallel = Number(process.env.PUBLISH_PARALLEL) || Number(process.env.PARALLEL) || 8,
  handleErrors = true
} = {}) => {
  const connector = new _kinesis2.default({ debug, streamName });

  const toInputParams = batchUow => _extends({}, batchUow, {
    inputParams: {
      Records: batchUow.batch.map(uow => toRecord(uow[eventField]))
    }
  });

  const putRecords = batchUow => {
    const p = connector.putRecords(batchUow.inputParams).then(publishResponse => _extends({}, batchUow, { publishResponse })).catch((0, _faults.rejectWithFault)(batchUow, !handleErrors));

    return (0, _highland2.default)(p); // wrap promise in a stream
  };

  return s => s.map(adornStandardTags(eventField)).batch(batchSize).map(_batch.toBatchUow).map(toInputParams).map(putRecords).parallel(parallel).flatMap(_batch.unBatchUow); // for cleaner logging and testing
};

const toRecord = exports.toRecord = e => ({
  Data: Buffer.from(JSON.stringify(e)),
  PartitionKey: e.partitionKey
});

const adornStandardTags = exports.adornStandardTags = eventField => uow => _extends({}, uow, {
  event: _extends({}, uow[eventField], {
    tags: _extends({}, envTags(uow.pipeline), (0, _filters.skipTag)(), uow[eventField].tags)
  })
});

const envTags = exports.envTags = pipeline => ({
  account: process.env.ACCOUNT_NAME || 'undefined',
  region: process.env.AWS_REGION || /* istanbul ignore next */'undefined',
  stage: process.env.SERVERLESS_STAGE || 'undefined',
  source: process.env.SERVERLESS_PROJECT || 'undefined',
  functionname: process.env.AWS_LAMBDA_FUNCTION_NAME || 'undefined',
  pipeline: pipeline || 'undefined'
});

// testing
const toKinesisRecords = exports.toKinesisRecords = events => ({
  Records: events.map((e, i) => ({
    eventSource: 'aws:kinesis',
    // eventVersion: '1.0',
    eventID: `shardId-000000000000:${i}`,
    // eventName: 'aws:kinesis:record',
    // invokeIdentityArn: 'arn:aws:iam::123456789012:role/lambda-role',
    awsRegion: 'us-west-2',
    // eventSourceARN: 'arn:aws:kinesis:us-west-2:123456789012:stream/lambda-stream',
    kinesis: {
      // kinesisSchemaVersion: '1.0',
      // partitionKey: e.partitionKey,
      sequenceNumber: `${i}`,
      data: Buffer.from(JSON.stringify(e)).toString('base64')
      // approximateArrivalTimestamp: 1545084650.987,
    }
  }))
});